unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TMainForm = class(TForm)
    GroupBox1: TGroupBox;
    CardCombo: TComboBox;
    GoButton: TButton;
    InGroup: TGroupBox;
    OutGroup: TGroupBox;
    ReadTime: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure GoButtonClick(Sender: TObject);
    procedure ReadTimeTimer(Sender: TObject);
    procedure OutCheckClick(Sender: TObject);
  protected
    InCheck, OutCheck: array[0..11] of TCheckBox;
    FETData: LongWord;
    CardNum: LongInt;

    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  AIOWDM;

function InStrC(C: Char; S: String; StartIndex: Integer = 1): LongWord;
var
  I: Integer;

begin
  for I := StartIndex to Length(S) do
    if S[I] = C then begin
      Result := I;
      Exit;
    end
  ;

  Result := 0;
end;

function InStrRC(C: Char; S: String; StartIndex: Integer = MAXINT): LongWord;
var
  I: Integer;

begin
  I := Length(S);
  if StartIndex > I then StartIndex := I;
  for I := StartIndex downto 1 do
    if S[I] = C then begin
      Break;
    end
  ;

  Result := I;
end;

procedure LoadDefaultFont(IntoFont: TFont);
var
  FontSize: Integer;
  NCM: TNonClientMetrics;

begin
  NCM.cbSize := SizeOf(NCM);
  if SystemParametersInfo(SPI_GETNONCLIENTMETRICS, SizeOf(NCM), @NCM, 0) then begin
    FontSize := Round(-3/4 * NCM.lfMessageFont.lfHeight);
    IntoFont.Name := NCM.lfMessageFont.lfFaceName;
  end
  else begin
    FontSize := Abs(IntoFont.Size);
    IntoFont.Name := 'Tahoma';
  end;
  IntoFont.PixelsPerInch := 72;
  IntoFont.Size := FontSize;
end;



procedure TMainForm.FormCreate(Sender: TObject);
var
  iCardNum: LongInt;
  DeviceID: LongWord;
  Nom: AnsiString;
  NomSize: LongWord;

begin
  for iCardNum := 0 to GetNumCards-1 do begin
    NomSize := 256;
    SetString(Nom, nil, NomSize);
    if QueryCardInfo(iCardNum, @DeviceID, nil, @NomSize, @Nom[1]) = 0 then Continue;
    if DeviceID <> $0FC0 then Continue;

    if NomSize > 0 then Dec(NomSize);
    SetLength(Nom, NomSize);

    CardCombo.Items.Add(Nom + ' @' + IntToStr(iCardNum));
    CardCombo.ItemIndex := 0;
    GoButton.Enabled := True;
    ActiveControl := GoButton;
  end;
end;

procedure TMainForm.GoButtonClick(Sender: TObject);
var
  I: Integer;
  CardStr: String;

begin
  I := CardCombo.ItemIndex;
  if I < 0 then Exit;

  CardStr := CardCombo.Items[I];
  I := InStrRC('@', CardStr);
  CardNum := StrToInt(Copy(CardStr, I+1, MAXINT));

  CardCombo.Enabled := False;
  GoButton.Enabled := False;
  for I := 0 to 11 do begin
    InCheck[I] := TCheckBox.Create(Self);
    InCheck[I].Enabled := False;
    InCheck[I].SetBounds(9999, 16, 40, 16);
    InCheck[I].Caption := IntToStr(I);
    InCheck[I].Align := alLeft;
    InCheck[I].Parent := InGroup;

    OutCheck[I] := TCheckBox.Create(Self);
    OutCheck[I].SetBounds(9999, 16, 40, 16);
    OutCheck[I].Tag := I;
    OutCheck[I].Caption := IntToStr(I);
    OutCheck[I].Align := alLeft;
    OutCheck[I].Parent := OutGroup;
  end;

  FETData := RelInPortL(CardNum, 0) and $FFFFFF;
  for I := 0 to 11 do OutCheck[I].Checked := (FETData and (1 shl I)) <> 0;
  for I := 0 to 11 do OutCheck[I].OnClick := OutCheckClick;

  ReadTime.Enabled := True;
  ReadTimeTimer(ReadTime);
end;

procedure TMainForm.ReadTimeTimer(Sender: TObject);
var
  IIData: LongWord;
  I: Integer;

begin
  IIData := RelInPortL(CardNum, 4);
  for I := 0 to 11 do InCheck[I].Checked := (IIData and (1 shl I)) <> 0;
end;

procedure TMainForm.OutCheckClick(Sender: TObject);
var
  iBit: Integer;

begin
  iBit := (Sender as TControl).Tag;
  if OutCheck[iBit].Checked then
    FETData := FETData or (1 shl iBit)
  else
    FETData := FETData and not (1 shl iBit)
  ;
  RelOutPortL(CardNum, 0, FETData);
end;

end.
